//====================================================== file = outport.c =====
//=  Output port for VOQ switch simulation model                              =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: outport() - output port                                        =
//=---------------------------------------------------------------------------=
//=  History:  KJC/KY (02/20/01) - Genesis (from voq_1 1.00)                  =
//=            KY (3/14/01) - Fixed PAUSE                                     =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff
#include <assert.h>     // Needed for assert() macro

void outport(int from_id, int out_id, int pkt_len, int pkt_type, int tail_flag,
             double org_time)
{
  create("outport");

  // Decrement counter semphores for a cell segmenter and outport pair
  Inport_count[from_id]--;
  Outport_count[out_id]--;

  // Clear busy events and set transmit done event
  clear(Inport_busy[from_id]);
  clear(Outport_busy[out_id]);
  set(Xmit_done[out_id]);

  // When the last cell for a segmented packet arrives, the cells are considered
  // reassembled and the packet is transmitted on the output link.
  if (tail_flag == TRUE)
  {
    reserve(Outport_buf[out_id]);
    hold((8.0 * pkt_len) / MEDIA_RATE);
    release(Outport_buf[out_id]);

    // Record statistics for packet leaving this port
    if (pkt_type == 0) record((clock - org_time), Delay_table_0[out_id]);
    if (pkt_type == 1) record((clock - org_time), Delay_table_1[out_id]);
    record(pkt_len, Bytes_table[out_id]);
  }
  return;
}
