//========================================================= file = link.c =====
//=  Link for propagation delay for VOQ switch simulation model               =
//=============================================================================
//=  Notes:  1) Note the all time units are in milliseconds                   =
//=---------------------------------------------------------------------------=
//=  Contains: send_pkt() - Downstream link for sending a packet              =
//=            send_pause() - Upstream link for sending a PAUSE               =
//=---------------------------------------------------------------------------=
//=  History:  KJC/KY (02/20/01) - Genesis (from voq_1 1.00)                  =
//=            KY (3/14/01) - Fixed PAUSE                                     =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

//=============================================================================
//==  Downstream link for sending a packet                                   ==
//=============================================================================
void send_pkt(int in_id, int out_id, int pkt_len, int pkt_type, double org_time)
{
  create("send_pkt");

  // Hold for a propagation delay
  hold((double) LINK_LEN * PROP_SPEED);

  // Deliver the packet to the segmenter
  seg[in_id](in_id, out_id, pkt_len, pkt_type, org_time);
}

//=============================================================================
//==  Upstream link for sending a PAUSE                                      ==
//==   - Exploits global variables for controlling source                    ==
//=============================================================================
void send_pause(int in_id, int pause_flag)
{
  create("send_pause");

  // Hold for a propagation delay to get the PAUSE to the source
  hold((double) LINK_LEN * PROP_SPEED);

  // Increment Pause_count
  Pause_count[in_id]++;

  // Set PAUSE state in generator
  if (pause_flag == XOFF)
    Pause_recv[in_id] = XOFF;
  else
  {
    Pause_recv[in_id] = XON;
    if (wait_cnt(Pause_event[in_id]) > 0) set(Pause_event[in_id]);
  }
}
